using HermesSocketLibrary.db;
using HermesSocketServer.Models;
using HermesSocketServer.Services;
using ILogger = Serilog.ILogger;

namespace HermesSocketServer.Requests
{
    public class UpdateTTSUser : IRequest
    {
        public string Name => "update_tts_user";
        public string[] RequiredKeys => ["chatter", "voice"];
        private ChannelManager _channels;
        private Database _database;
        private readonly ServerConfiguration _configuration;
        private ILogger _logger;

        public UpdateTTSUser(ChannelManager channels, Database database, ServerConfiguration configuration, ILogger logger)
        {
            _database = database;
            _channels = channels;
            _configuration = configuration;
            _logger = logger;
        }


        public async Task<RequestResult> Grant(string sender, IDictionary<string, object>? data)
        {
            if (long.TryParse(data["chatter"].ToString(), out long chatterId))
                data["chatter"] = chatterId;
            data["voice"] = data["voice"].ToString();
            data["user"] = sender;

            var check = await _database.ExecuteScalar("SELECT state FROM \"TtsVoiceState\" WHERE \"userId\" = @user AND \"ttsVoiceId\" = @voice", data) ?? false;
            if ((check is not bool state || !state) && chatterId != _configuration.Tts.OwnerId)
                return RequestResult.Failed("Voice is either non-existent or disabled on this channel.");

            var channel = _channels.Get(sender);
            var result = channel.Chatters.Set(chatterId.ToString(), new ChatterVoice()
            {
                UserId = sender,
                ChatterId = chatterId,
                VoiceId = data["voice"].ToString()!
            });
            if (result)
            {
                _logger.Information($"Updated chatter's [chatter: {data["chatter"]}] selected tts voice [voice: {data["voice"]}] in channel [channel: {sender}]");
                return RequestResult.Successful(null);
            }
            return RequestResult.Failed("Soemthing went wrong when updating the cache.");
        }
    }
}